SystemOrganization addCategory: #'Pier-ExternalLogin'!

!PRPierConfiguration methodsFor: '*pier-externallogin' stamp: 'kph 8/16/2007 22:45'!
attributeLoginValidationClass
	^ (WAListAttribute key: #loginValidation group: #pier)
		options: PUExternalValidationPlugin withAllSubclasses;
		yourself! !

Object subclass: #PUExternalValidationPlugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-ExternalLogin'!

!PUExternalValidationPlugin commentStamp: 'kph 8/17/2007 01:43' prior: 0!
My subclasses provide the connection between the pier Login (PUELogin) and external validation
resources, e.g. LDAP or a MySql data table of user accounts.

Our model for logging in consists of two databases:

1. a validation 'accountDetails' database, which handles the user/password look up and is
looked after by the IT/site infrastucture. 

2. a domain 'userData' database which holds user information and is 
managed by the application client.

Plugins implement the following API.

MyValidationPlugin-class-loginFor: aLoginCommand

	returns an 'accountDetails' object for the user, or nil if the validation failed.

MyValidationPlugin-class-userFrom: aCommand details: details

	Default behaviour picks a user (named by accountDetails-#pierRole from the pier-internal users to use as a template for the external user.
	The user object itself is only for this session, it does not get permanently added to the internal kernel users table.

MyValidationPlugin-class-userDataFor: user with: accountDetails

	returns a 'user data' domain object for the user for use in the application.
	This 'user data' domain object is given a final chance to refuse login, it can throw an exception
	in its userData-#validateLogin (this allows the domain database to include such things as an account expiry date etc)
	
PULoginWithExternalValidation puts accountDetails and userData objects into properties in the PUUser object for this session.	
	
	

!

!PUExternalValidationPlugin class methodsFor: 'as yet unclassified' stamp: 'kph 8/16/2007 22:54'!
forLogin: aLoginCommand

	^ nil! !

!PUExternalValidationPlugin class methodsFor: 'as yet unclassified' stamp: 'kph 8/16/2007 23:14'!
invalidLoginRole: role notFoundForUser: username
	PUInvalidLogin
		description: self description
		signal: 'Role: ', role, ' for user: ', (username ifNil: [ 'not-as-yet-defined' ]), ' not found'
! !

!PUExternalValidationPlugin class methodsFor: 'as yet unclassified' stamp: 'kph 8/16/2007 23:46'!
loginDescription: desc

	" you can annotate the login description, see Magritte-ComponentDecoration"
	
	^ desc! !

!PUExternalValidationPlugin class methodsFor: 'as yet unclassified' stamp: 'kph 8/16/2007 23:17'!
userDataFor: user with: accountDetails

	 ! !

!PUExternalValidationPlugin class methodsFor: 'as yet unclassified' stamp: 'kph 8/16/2007 23:20'!
userFrom: aCommand details: details

	| prototypeUserName prototype |
	
	prototypeUserName := details pierRole.
	 
	prototype := aCommand kernel users detect: [ :userPrototype  | userPrototype name = prototypeUserName ] 
								ifNone: [ self invalidLoginRole: details pierRole notFoundForUser: details username ].
							 
	^ ((prototype copy) 
				setName: details username;
				yourself)
	! !

PULogin subclass: #PUELogin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-ExternalLogin'!

!PUELogin commentStamp: 'kph 8/16/2007 23:46' prior: 0!
I validate the login request against the in-image pier users, and then against an external resource. See class comment for PUExternalValidationPlugin

Returning an object which contains the retrieved database record for that user we instanciate a 
temporary pier user from it, using an existing 'role-user' as a prototype.

The roles defined in the db refer to internal user prototypes prefixed with 'role-' in order to prevent 
accidentally (on purpose) spoofing of an in-image only admin user.

This allows the access priviliges to be managed entirely within pier using a manageable number of users, which is handy given that there are many thousand accounts defined in the users validation table.

The chosen plugin also gets the opportunity to annotate my description (see Magritte-ComponentDecoration)!

!PUELogin class methodsFor: 'as yet unclassified' stamp: 'kph 8/16/2007 22:25'!
isAbstract

	^ false! !

!PUELogin class methodsFor: 'as yet unclassified' stamp: 'kph 8/9/2007 03:18'!
label
	^ 'Login'! !

!PUELogin methodsFor: 'accessing' stamp: 'kph 8/16/2007 23:39'!
description

	^ self validationPlugin loginDescription: super description! !

!PUELogin methodsFor: 'accessing' stamp: 'kph 8/17/2007 01:38'!
doExternalValidation

	 | validation userData accountDetails loggedinUser | 
		 
	validation := self validationPlugin.	
		
	accountDetails := (validation forLogin: self) ifNil: [ super invalidLogin ].
			
	loggedinUser := validation userFrom: self details: accountDetails.
	loggedinUser propertyAt: #accountDetails put: accountDetails.
			
	userData := validation userDataFor: loggedinUser with: accountDetails. 
 
	loggedinUser propertyAt: #userData put: userData.

	userData ifNotNilDo: [ :d | d validateLogin ].
	
	^ loggedinUser

		
	! !

!PUELogin methodsFor: 'accessing' stamp: 'kph 8/16/2007 16:01'!
invalidLogin

	^ self doExternalValidation! !

!PUELogin methodsFor: 'accessing' stamp: 'kph 8/16/2007 22:53'!
validationPlugin
	
	^ self context session application preferenceAt: #loginValidation.
	
	
	! !

!PULogin class methodsFor: '*pier-externallogin-override' stamp: 'kph 8/17/2007 00:04'!
isAbstract

	"extemely cheeky"
	^ true! !
